# -*- coding:Utf-8 -*-
import es, re, psyco, dm_python
psyco.full()


admin = set()
command = set()
importname = __import__(__name__)
regex = re.compile('//.*|(\r|\n|")+')
cfgpath = es.getString("eventscripts_gamedir") + "/cfg/dm_python/"

class DmConfig(object):
    """ Config Management """
    def __init__(self):
        self.steamid = es.ServerVar("dp_admin_steamid", "", "[dm_python] administrator steamid")
        self.admin = es.ServerVar("dp_admin_command", "", "[dm_python] administration menu command")
        self.command = es.ServerVar("dp_menu_command", "", "[dm_python] deathmatch menu command")
        self.style = es.ServerVar("dp_menu_style", "", "[dm_python] deathmatch menu style")
        self.timeout = es.ServerVar("dp_menu_timeout", "", "[dm_python] menu time out")
        self.plugin = es.ServerVar("dp_respawn_plugin", "", "[dm_python] plugin used for respawn players")
        self.protection = es.ServerVar("dp_respawn_protection", "", "[dm_python] protect player when he spawn")
        self.random = es.ServerVar("dp_respawn_random", "", "[dm_python] respawn player at his team spawn")
        self.respawn = es.ServerVar("dp_respawn_time", "", "[dm_python] time for respawn players")
        self.dissolver = es.ServerVar("dp_ragdoll_dissolver", "", "[dm_python] dissolve ragdoll")
        self.extension = es.ServerVar("dp_extended_config", "", "[dm_python] extended configuration")

        self.assaultsuit = es.ServerVar("dp_give_assaultsuit", "", "[dm_python] give assaultsuit")
        self.nightvision = es.ServerVar("dp_give_nightvision", "", "[dm_python] give nightvision")
        self.hegrenade = es.ServerVar("dp_hegrenade_max", "", "[dm_python] hegrenade max")
        self.flashbang = es.ServerVar("dp_flashbang_max", "", "[dm_python] flashbang max")
        self.smokegrenade = es.ServerVar("dp_smokegrenade_max", "", "[dm_python] smokegrenade max")
        self.botPrimary = es.ServerVar("dp_bot_primary", "", "[dm_python] primary weapon allowed for bot")
        self.botSecondary = es.ServerVar("dp_bot_secondary", "", "[dm_python] secondary weapon allowed for bot")

        self.kills = es.ServerVar("dp_vote_kills", "", "[dm_python] activate votemap")
        self.time = es.ServerVar("dp_vote_time", "", "[dm_python] time for vote")
        self.maxmap = es.ServerVar("dp_vote_maxmap", "", "[dm_python] number of map added in vote")
        self.lastmaps = es.ServerVar("dp_vote_lastmaps", "", "[dm_python] don't add last maps played in vote")
        self.startsound = es.ServerVar("dp_vote_startsound", "", "[dm_python] sound played at vote begening")
        self.endsound = es.ServerVar("dp_vote_endsound", "", "[dm_python] sound played when vote is ending")

        self.reloadclip = es.ServerVar("dp_extension_reload_clip", "", "[dm_python] reload clip when player make a kill")
        self.giveammo = es.ServerVar("dp_extension_give_ammo", "", "[dm_python] give ammo when munitions are emptie")

    def __contains__(self, item):
        return item in map(lambda steamid: steamid.strip(), str(self.steamid).split(","))

    def __getitem__(self, item):
        if item == "admin": return (self.admin, None, "config")
        elif item == "command": return (self.command, None, "config")
        elif item == "style": return (self.style, ("1", "2"), "config")
        elif item == "timeout": return (self.timeout, 0, "config")
        elif item == "plugin":
            plugin = ["es"]
            if dm_python.SourcePythonExtensions: plugin.append("spe")
            #if dm_python.NativeTools: plugin.append("nt")
            return (self.plugin, tuple(plugin), "config")
        elif item == "protection": return (self.protection, 0, "config")
        elif item == "random": return (self.random, ("0", "1"), "config")
        elif item == "respawn": return (self.respawn, 1, "config")
        elif item == "dissolver": return (self.dissolver, ("0", "1"), "config")
        elif item == "extension": return (self.extension, ("0", "1"), "config")

        elif item == "assaultsuit": return (self.assaultsuit, ("0", "1"), "items")
        elif item == "nightvision": return (self.nightvision, ("0", "1"), "items")
        elif item == "hegrenade": return (self.hegrenade, 0, "items")
        elif item == "flashbang": return (self.flashbang, 0, "items")
        elif item == "smokegrenade": return (self.smokegrenade, 0, "items")

        elif item == "kills": return (self.kills, 0, "votemap")
        elif item == "time": return (self.time, 0, "votemap")
        elif item == "maxmap": return (self.maxmap, 0, "votemap")
        elif item == "lastmaps": return (self.lastmaps, 0, "votemap")

        elif item == "reloadclip": return (self.reloadclip, ("0", "1", "2"), "extension")
        elif item == "giveammo": return (self.giveammo, ("0", "1"), "extension")

    def update(self):
        self.execCfg()
        self.createCmd()

    def execCfg(self):
        es.mexec("dm_python/config.cfg")
        es.mexec("dm_python/items.cfg")
        es.mexec("dm_python/vote/votemap.cfg")

        if str(self.plugin) == "spe" and not dm_python.SourcePythonExtensions:
            self.plugin.set("es")
            self.saveCfg("config")
            es.dbgmsg(0, 'dm_python: cannot use plugin Source Python Extensions, "dp_respawn_plugin" have been modified to "es"')

        if int(self.extension):
            if not dm_python.extentionLoaded(): es.load("dm_python/dm_extension")
        else:
            if dm_python.extentionLoaded(): es.unload("dm_python/dm_extension")

        if bool(self.startsound): es.stringtable("downloadables", "sounds/" + str(self.startsound))
        if bool(self.endsound): es.stringtable("downloadables", "sounds/" + str(self.endsound))

    def saveCfg(self, save = "all"):
        if save == "all":
            self.saveCfg("config")
            self.saveCfg("items")
            self.saveCfg("votemap")
            self.saveCfg("extension")

        elif save == "config":
            try:
                openfile = open(cfgpath + "config.cfg", "w")
                openfile.write(
                    "// config.cfg for dm_python script\n\n\n" +
                    '// X = administrator steamid\ndp_admin_steamid "%s"\n\n' % self.steamid +
                    '// X = administration menu command\ndp_admin_command "%s"\n\n' % self.admin +
                    '// X = deathmatch menu command\ndp_menu_command "%s"\n\n' % self.command +
                    '// X = deathmatch menu style (1|2)\ndp_menu_style "%s"\n\n' % self.style +
                    '// X = close menu after X second, 0 = do not close automatically\ndp_menu_timeout "%s"\n\n' % self.timeout +
                    '// X = plugin used for respawn a player, "es" (EventScripts) or "spe" (SourcePythonExtensions)\ndp_respawn_plugin "%s"\n\n' % self.plugin +
                    '// X = time in second for respawn a player (1 min)\ndp_respawn_time "%s"\n\n' % self.respawn +
                    '// X = protect player when he spawn (in second)\ndp_respawn_protection "%s"\n\n' % self.protection +
                    '// 0 = respawn player at his team spawn, 1 = random spawn team\ndp_respawn_random "%s"\n\n' % self.random +
                    '// 0 = no effect at ragdoll removing, 1 = dissolve effect\ndp_ragdoll_dissolver "%s"\n\n' % self.dissolver +
                    '// 0 = disable extended configuration, 1 = enable (see ../cfg/dm_python/extension.cfg)\ndp_extended_config "%s"' % self.extension)
                openfile.close()
            except IOError: es.dbgmsg(0, "dm_python: cannot read file cfg/dm_python/config.cfg")

        elif save == "items":
            try:
                openfile = open(cfgpath + "items.cfg", "w")
                openfile.write(
                    "// items.cfg for dm_python script\n\n\n" +
                    '// 1 = give kevlar and helmet\ndp_give_assaultsuit "%s"\n\n' % self.assaultsuit +
                    '// 1 = give nightvision\ndp_give_nightvision "%s"\n\n' % self.nightvision +
                    '// X = hegrenade max\ndp_hegrenade_max "%s"\n\n' % self.hegrenade +
                    '// X = flashbang max\ndp_flashbang_max "%s"\n\n' % self.flashbang +
                    '// X = smokegrenade max\ndp_smokegrenade_max "%s"\n\n' % self.smokegrenade +
                    '// X = allowed primary weapons for bots, "" = all, 0 = no primary weapon\ndp_bot_primary "%s"\n\n' % self.botPrimary +
                    '// X = allowed secondary weapons for bots, "" = all, 0 = no secondary weapon\ndp_bot_secondary "%s"' % self.botSecondary)
                openfile.close()
            except IOError: es.dbgmsg(0, "dm_python: cannot read file cfg/dm_python/items.cfg")

        elif save == "votemap":
            try:
                openfile = open(cfgpath + "vote/votemap.cfg", "w")
                openfile.write(
                    "// votemap.cfg for dm_python script\n\n\n" +
                    '// X = submit votemap after X kill, 0 = disable votemap\ndp_vote_kills "%s"\n\n' % self.kills +
                    '// X = time for vote (sec)\ndp_vote_time "%s"\n\n' % self.time +
                    '// X = number of map added into the vote\ndp_vote_maxmap "%s"\n\n' % self.maxmap +
                    '// X = don\'t add the X last maps played into the vote\ndp_vote_lastmaps "%s"\n\n' % self.lastmaps +
                    '// X = sound played at vote beginning\ndp_vote_startsound "%s"\n\n' % self.startsound +
                    '// X = sound played when vote is ended\ndp_vote_endsound "%s"' % self.endsound)
                openfile.close()
            except IOError: es.dbgmsg(0, "dm_python: cannot read file cfg/dm_python/vote/votemap.cfg")

        elif save == "extension":
            try:
                openfile = open(cfgpath + "extension.cfg", "w")
                openfile.write(
                    "// extension.cfg for dm_python script\n\n\n" +
                    '// 2 = reload clip when player make a kill, 1 = only by headshot, 0 = off\ndp_extension_reload_clip "%s"\n\n' % self.reloadclip +
                    '// 1 = give ammo when munitions are emptie, 0 = off\ndp_extension_give_ammo "%s"' % self.giveammo)
                openfile.close()
            except IOError: es.dbgmsg(0, "dm_python: cannot read file cfg/dm_python/votemap.cfg")

    def createCmd(self, cmd = "all"):
        if cmd == "all":
            self.createCmd("command")
            self.createCmd("admin")

        elif cmd == "command":
            self.deleteCmd("command")
            for word in map(lambda word: word.strip(), str(self.command).split(",")):
                command.add(word)
                es.regsaycmd(word, "dm_python/deathmatchMenu", "[dm_python] deathmatch menu acces")
                es.regclientcmd(word, "dm_python/deathmatchMenu", "[dm_python] deathmatch menu acces")

        elif cmd == "admin":
            self.deleteCmd("admin")
            for word in map(lambda word: word.strip(), str(self.admin).split(",")):
                admin.add(word)
                es.regsaycmd(word, "dm_python/administrationMenu", "[dm_python] administration menu acces")
                es.regclientcmd(word, "dm_python/administrationMenu", "[dm_python] administration menu acces")

    def deleteCmd(self, cmd = "all"):
        if cmd == "all":
            self.deleteCmd("command")
            self.deleteCmd("admin")

        elif cmd == "command":
            for word in command:
                es.unregsaycmd(word)
                es.unregclientcmd(word)
            command.clear()

        elif cmd == "admin":
            for word in admin:
                es.unregsaycmd(word)
                es.unregclientcmd(word)
            admin.clear()

# create DmConfig object
dmConfig = DmConfig()
