# -*- coding:Utf-8 -*-
import es, psyco, random, gamethread, langlib, playerlib, simplemenulib, dm_python
from dm_config import dmConfig
from dm_weapon import dmWeapon
from dm_spawns import dmSpawns
psyco.full()


lastgive = es.ServerVar("eventscripts_lastgive")
lastweapon = {}
dmPlayer = {}

class DmPlayer(object):
    """ Player Management """
    def __init__(self, userid):
        self.primary = None
        self.secondary = None
        self.hegrenade = 0
        self.flashbang = 0
        self.smokegrenade = 0

        self.indexspawn = -1
        self.firstspawn = True
        self.userid = int(userid)
        self.struserid = str(userid)
        self.isbot = es.isbot(userid)
        self.player = playerlib.getPlayer(userid)
        userlang = es.getclientvar(userid, "cl_language")
        self.lang = langlib.getLangAbbreviation(userlang) if userlang else "en"

        self.threadProtection =  "_dm_python_protection_" + self.struserid
        self.threadNoBlock = "_dm_python_noblock_" + self.struserid
        self.threadWeapons = "_dm_python_weapons_" + self.struserid
        self.threadItems = "_dm_python_items_" + self.struserid

        self.entitiesWeaponsName = "_dm_entities_weapons_" + self.struserid
        self.entitiesItemsName = "_dm_entities_items_" + self.struserid
        self.dissolverRagDoll = "_dp_dissolver_ragdoll_" + self.struserid
        es.server.queuecmd("es_xcreateentity env_entity_dissolver " + self.dissolverRagDoll)

    def __del__(self):
        gamethread.cancelDelayed(self.threadNoBlock)
        gamethread.cancelDelayed(self.threadProtection)
        gamethread.cancelDelayed(self.threadWeapons)
        gamethread.cancelDelayed(self.threadItems)

        if es.exists("userid", self.userid):
            self.player.refreshAttributes()
            if not self.player.isdead:
                self.player.godmode = False
                self.player.color = (255, 255, 255, 255)

        else:
            userid = es.getuserid()
            if userid:
                es.server.queuecmd(
                    "es_xfire %s %s kill;" % (userid, self.entitiesWeaponsName) +
                    "es_xfire %s %s kill;" % (userid, self.entitiesItemsName) +
                    "es_xfire %s %s kill" % (userid, self.dissolverRagDoll))

    def __getitem__(self, name):
        if name == "steamid":
            steamid = es.getplayersteamid(self.userid)
            if steamid in "BOTSTEAM_ID_PENDINGSTEAM_ID_LAN": return None
            else:
                if not lastweapon.has_key(steamid): lastweapon[steamid] = [None, None]
                return steamid

        elif name == "lastprimary":
            steamid = self["steamid"]
            if steamid is None: return None
            else:
                try: return lastweapon[steamid][0]
                except KeyError: return None

        elif name == "lastsecondary":
            steamid = self["steamid"]
            if steamid is None: return None
            else:
                try: return lastweapon[steamid][1]
                except KeyError: return None

    def __setitem__(self, name, value):
        if name == "lastprimary":
            steamid = self["steamid"]
            if steamid is not None:
                if lastweapon.has_key(steamid): lastweapon[steamid][0] = value

        elif name == "lastsecondary":
            steamid = self["steamid"]
            if steamid is not None:
                if lastweapon.has_key(steamid): lastweapon[steamid][1] = value

    def i_noblock(self):
        gamethread.cancelDelayed(self.threadNoBlock)
        sx, sy, sz = self.player.location
        status = 5

        for userid in es.getUseridList():
            if userid <> self.userid:
                ux, uy, uz = es.getplayerlocation(userid)
                if ((sx - ux) ** 2 + (sy - uy) ** 2 + (sz - uz) ** 2) ** 0.5 <= 180:
                    gamethread.delayedname(1, self.threadNoBlock, self.i_noblock)
                    status = 2
                    break
        es.setplayerprop(self.userid, "CBaseEntity.m_CollisionGroup", status)

    def i_protection(self):
        self.player.godmode = False
        self.player.color = (255, 255, 255, 255)

    def respawn(self):
        if es.getplayerprop(self.userid, "CBasePlayer.pl.deadflag"):
            plugin = str(dmConfig.plugin).lower()
            if plugin <> "es":
                if plugin == "spe" and not dm_python.SourcePythonExtensions: plugin = "es"
                #elif plugin == "nt" and not dm_python.NativeTools: plugin = "es"
            esmode = True if plugin == "es" else False

            if plugin == "spe":
                try: dm_python.es_spe.respawn(self.userid)
                except NameError:
                    es.dbgmsg(0, 'dm_python: cannot respawn userid "%s" with plugin "Source Python Extensions"' % self.userid)
                    es.dbgmsg(0, "dm_python: try to respawn the player with EventScripts only...")
                    esmode = True

            """
            elif plugin == "nt":
                try: dm_python.es_nt.spawnPlayer(self.userid)
                except NameError:
                    es.dbgmsg(0, 'dm_python: cannot respawn userid "%s" with plugin "Native Tools"' % self.userid)
                    es.dbgmsg(0, "dm_python: try to respawn the player with EventScripts only...")
                    esmode = True
            """

            if esmode:
                es.setplayerprop(self.userid, "CCSPlayer.m_iPlayerState", 0)
                es.setplayerprop(self.userid, "CCSPlayer.baseclass.m_lifeState", 512)
                es.server.queuecmd("es_xspawnplayer " + self.struserid)
            gamethread.queue(self._respawn)

    def _respawn(self):
        if es.getplayerprop(self.userid, "CBasePlayer.pl.deadflag"): raise dm_python.DmPythonError, 'respawning of userid "%s" failed' % self.userid
        spawn = dmSpawns.get(self.userid)
        if spawn is not None:
            ox, oy, oz, ax, ay, az, origin = spawn
            es.server.queuecmd("es_xsetpos %s %s %s %s" % (self.userid, ox, oy, oz))
            es.server.queuecmd("es_xsetang %s %s %s %s" % (self.userid, ax, ay, az))

    def spawn(self, team):
        self.player.refreshAttributes()
        if not self.player.isdead:
            es.server.queuecmd(
                "es_xcreateentity env_entity_dissolver %s;" % self.dissolverRagDoll +
                "es_entitysetvalue server_var(eventscripts_lastgive) dissolvetype %s;" % random.randint(0,3) +
                "es_entitysetvalue server_var(eventscripts_lastgive) target cs_ragdoll;" +
                "es_entitysetvalue server_var(eventscripts_lastgive) magnitude 100")

            # init grenades management
            self.player.he = 0
            self.player.fb = 0
            self.player.sg = 0
            self.hegrenade = 0
            self.flashbang = 0
            self.smokegrenade = 0

            # initialize noblock and protection system
            self.i_noblock()
            self.player.godmode = True
            self.player.color = (255, 255, 255, 120)
            gamethread.cancelDelayed(self.threadProtection)
            gamethread.delayedname(int(dmConfig.protection), self.threadProtection, self.i_protection)

            # remove primary and secondary weapons if they're not corresponding to player's selection
            primary, secondary = str(self.player.primary), str(self.player.secondary)
            if primary <> "0" and primary <> self.primary:
                es.server.queuecmd("es_xsetentityname %s %s" % (self.player.getWeaponIndex(primary), self.entitiesWeaponsName))
            if secondary <> "0" and secondary <> self.secondary:
                es.server.queuecmd("es_xsetentityname %s %s" % (self.player.getWeaponIndex(secondary), self.entitiesWeaponsName))
            gamethread.queue(self.remove, "weapons")

            # give weapons and items
            gamethread.cancelDelayed(self.threadItems)
            gamethread.cancelDelayed(self.threadWeapons)
            gamethread.delayedname(.1, self.threadItems, self.items)
            gamethread.delayedname(.2, self.threadWeapons, self.weapons)

    def items(self):
        # give items and grenades
        if int(dmConfig.assaultsuit): es.server.queuecmd("es_xgive %s item_assaultsuit" % self.userid)
        if int(dmConfig.nightvision): es.server.queuecmd("es_xgive %s item_nvgs" % self.userid)
        if int(dmConfig.hegrenade):
            es.server.queuecmd(
                "es_xgive %s weapon_hegrenade;" % self.userid +
                "es_setentityname server_var(eventscripts_lastgive) " + self.entitiesItemsName)
        if int(dmConfig.flashbang):
            es.server.queuecmd(
                "es_xgive %s weapon_flashbang;" % self.userid +
                "es_setentityname server_var(eventscripts_lastgive) " + self.entitiesItemsName)
        if int(dmConfig.smokegrenade):
            es.server.queuecmd(
                "es_xgive %s weapon_smokegrenade;" % self.userid +
                "es_setentityname server_var(eventscripts_lastgive) " + self.entitiesItemsName)

    def weapons(self):
        # give primary and secondary weapons to player
        if self.isbot:
            primary, secondary = dmWeapon.botPrimary(), dmWeapon.botSecondary()
            if primary is not None:
                es.server.queuecmd(
                    "es_xgive %s weapon_%s;" % (self.userid, primary) +
                    "es_setentityname server_var(eventscripts_lastgive) " + self.entitiesWeaponsName)
            if secondary is not None:
                es.server.queuecmd(
                    "es_xgive %s weapon_%s;" % (self.userid, secondary) +
                    "es_setentityname server_var(eventscripts_lastgive) " + self.entitiesWeaponsName)

        else:
            if self.primary is None:
                lastprimary = self["lastprimary"]
                if lastprimary is not None: self.primary = lastprimary
            if not self.primary in dmWeapon:
                try: self.primary = "weapon_" + random.choice(dmWeapon.primary)
                except IndexError: self.primary = None
                self["lastprimary"] = self.primary

            if self.secondary is None:
                lastsecondary = self["lastsecondary"]
                if lastsecondary is not None: self.secondary = lastsecondary
            if not self.secondary in dmWeapon:
                try: self.secondary = "weapon_" + random.choice(dmWeapon.secondary)
                except IndexError: self.secondary = None
                self["lastsecondary"] = self.secondary

            if self.secondary:
                es.server.queuecmd(
                    "es_xgive %s %s;" % (self.userid, self.secondary) +
                    "es_setentityname server_var(eventscripts_lastgive) %s;" % self.entitiesWeaponsName +
                    "es_xsexec %s use %s" % (self.userid, self.secondary))

            if self.primary:
                es.server.queuecmd(
                    "es_xgive %s %s;" % (self.userid, self.primary) +
                    "es_setentityname server_var(eventscripts_lastgive) %s;" % self.entitiesWeaponsName +
                    "es_xsexec %s use %s" % (self.userid, self.primary))

    def grenade(self, name):
        if not es.getplayerprop(self.userid, "CBasePlayer.pl.deadflag"):
            if name == "he":
                self.hegrenade += 1
                if self.hegrenade < int(dmConfig.hegrenade):
                    es.server.queuecmd(
                        "es_xgive %s weapon_hegrenade;" % self.userid +
                        "es_setentityname server_var(eventscripts_lastgive) " + self.entitiesItemsName)
            elif name == "fb":
                self.flashbang += 1
                if self.flashbang < int(dmConfig.flashbang):
                    es.server.queuecmd(
                        "es_xgive %s weapon_flashbang;" % self.userid +
                        "es_setentityname server_var(eventscripts_lastgive) " + self.entitiesItemsName)
            elif name == "sg":
                self.smokegrenade += 1
                if self.smokegrenade < int(dmConfig.smokegrenade):
                    es.server.queuecmd(
                        "es_xgive %s weapon_smokegrenade;" % self.userid +
                        "es_setentityname server_var(eventscripts_lastgive) " + self.entitiesItemsName)

    def remove(self, entitytype = "all"):
        if entitytype == "dissolver": es.server.queuecmd("es_xfire %s %s kill" % (self.userid, self.dissolverRagDoll))
        elif entitytype == "weapons": es.server.queuecmd("es_xfire %s %s kill" % (self.userid, self.entitiesWeaponsName))
        elif entitytype == "items": es.server.queuecmd("es_xfire %s %s kill" % (self.userid, self.entitiesItemsName))
        elif entitytype == "all": gamethread.queue(self.remove, "weapons"), gamethread.queue(self.remove, "items"), gamethread.queue(self.remove, "dissolver")

# Function for get/manage players
def player(userid):
    userid = int(userid)
    if not dmPlayer.has_key(userid): dmPlayer[userid] = DmPlayer(userid)
    return dmPlayer[userid]

def playerlist(filter = "all"):
    if filter == "all": return dmPlayer.keys()
    elif filter == "human": return [userid for userid in playerlist() if not dmPlayer[userid].isbot]

def unload():
    for userid in playerlist(): del dmPlayer[userid]
